//
//  GFaAreaInformationManageTableViewController.m
//
//  Copyright (c) 2014 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "GFaAreaInformationManageTableViewController.h"

#import <libGeoFenceManager/GeoFenceManager.h>
#import "GFaInformationDescriptionViewController.h"
#import "GFaUtility.h"

@interface GFaAreaInformationManageTableViewController () <UIAlertViewDelegate>

- (void)removeAreaInformation_:(NSString *)removeAreaID;
- (void)getAreaInformation_:(NSString *)getAreaID;

@end

@implementation GFaAreaInformationManageTableViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    if ( indexPath.section == 0 )
    {
        if ( indexPath.row == 1 )
        {
            NSString *title = @"エリア情報削除";
            NSString *message = @"エリアIDを入力して下さい";
            NSString *cancelButtonTitle = @"キャンセル";
            NSString *otherButtonTitle = @"OK";
            [GFaUtility showInputAlert:self title:title message:message text:@"" cancelButtonTitle:cancelButtonTitle otherButtonTitle:otherButtonTitle keyboardType:UIKeyboardTypeDefault otherHandler:^(NSString *text) {
                [self removeAreaInformation_:text];
            }];
        }
        else if ( indexPath.row == 2 )
        {
            NSString *title = @"エリア情報取得";
            NSString *message = @"エリアIDを入力して下さい.\nすべて取得する場合は,空欄のままOKを押して下さい.";
            NSString *cancelButtonTitle = @"キャンセル";
            NSString *otherButtonTitle = @"OK";
            [GFaUtility showInputAlert:self title:title message:message text:@"" cancelButtonTitle:cancelButtonTitle otherButtonTitle:otherButtonTitle keyboardType:UIKeyboardTypeDefault otherHandler:^(NSString *text) {
                [self getAreaInformation_:text];
            }];
        }
    }
}

#pragma mark Private

- (void)removeAreaInformation_:(NSString *)removeAreaID
{
    NSString *title = nil;
    NSString *mess = nil;
    GeoFenceManager *manager = [GeoFenceManager defaultManager];
    if ( removeAreaID.length > 0
        && [manager removeAreaInformation:@[removeAreaID]].count > 0 )
    {
        title = @"削除成功";
    }
    else
    {
        title = @"削除失敗";
        mess = [NSString stringWithFormat:@"エリアを削除できませんでした\n(エリアID: %@)", removeAreaID];
    }
    [GFaUtility showInformationAlert:self title:title message:mess];
}

- (void)getAreaInformation_:(NSString *)getAreaID
{
    
    GeoFenceManager *manager = [GeoFenceManager defaultManager];
    NSArray *info = nil;
    if ( getAreaID.length > 0 )
    {
        info = [manager areaInformationList:@[getAreaID]];
    }
    else
    {
        info = [manager areaInformationList:nil];
    }
    
    if ( info.count > 0 )
    {
        /* 該当する情報があった */
        GFaInformationDescriptionViewController *vc =
        [[GFaInformationDescriptionViewController alloc] initWithNibName:@"GFaInformationDescriptionViewController" bundle:nil];
        vc.title = @"エリア情報表示";
        vc.descriptionInfo = info;
        [self.navigationController pushViewController:vc animated:YES];
    }
    else
    {
        /* 該当なし */
        [GFaUtility showInformationAlert:self title:@"該当するエリア情報はありません" message:@""];
    }
}

@end

