//
//  GFaGeoFenceManagerSettingViewController.m
//
//  Copyright (c) 2013 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "GFaGeoFenceManagerSettingViewController.h"

#import <libGeoFenceManager/GeoFenceManager.h>
#import "GFaGeoFenceManagerSettingModel.h"
#import "GFaUtility.h"
#import "GFaCommon.h"

/* セクション定数定義 */
enum {
    Section_GeoFenceLibrary = 0 /* ジオフェンスライブラリ認証用 */
};

/* フィールド定数定義(ジオフェンスライブラリ認証用セクション) */
enum {
    Field_ApiKey = 0,   /* APIキー */
    Field_ClientId,     /* クライアントID */
    Field_Secret,       /* 秘密鍵 */
};

@interface GFaGeoFenceManagerSettingViewController ()

- (UITextField *)findTextField_:(NSInteger)section row:(NSInteger)row;
- (void)setValueToGeoFenceManagerSetting_;
- (void)setValueFromGeoFenceManagerSetting_;
- (void)setupAuthDevMenu_;

@end

@implementation GFaGeoFenceManagerSettingViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.title = @"ジオフェンス・ライブラリ設定";

    [self setValueFromGeoFenceManagerSetting_];
    [self setupAuthDevMenu_];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
    [self setValueToGeoFenceManagerSetting_];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/* 初期化ボタン押下 */
- (IBAction)push_Default:(id)sender
{
    GFaGeoFenceManagerSettingModel *model = [GFaGeoFenceManagerSettingModel defaultModel];
    
    [model resetSetting];
    [model saveToUserDefaults];
    [self setValueFromGeoFenceManagerSetting_];
    [self setupAuthDevMenu_];
}

#pragma mark UITableViewDelegate

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];

    __block UITextField *textField = [self findTextField_:indexPath.section row:indexPath.row];

    if (textField != nil)
    {
        [GFaUtility showInputAlert:self title:textField.accessibilityLabel message:@"" text:textField.text cancelButtonTitle:@"キャンセル" otherButtonTitle:@"確定" keyboardType:textField.keyboardType otherHandler:^(NSString *value) {
            textField.text = value;
        }];
    }
}

#pragma mark Private

/* 入力フィールド取得 */
- (UITextField *)findTextField_:(NSInteger)section row:(NSInteger)row
{
    switch (section) {
        case Section_GeoFenceLibrary:
            switch (row) {
                case Field_ApiKey:
                    return self.apikeyTextField;
                case Field_ClientId:
                    return self.clientTextField;
                case Field_Secret:
                    return self.secretTextField;
                default:
                    break;
            }
        default:
            break;
    }
    return nil;
}

/* 画面入力値を設定保持クラスに設定 */
- (void)setValueToGeoFenceManagerSetting_
{
    GFaGeoFenceManagerSettingModel *settingModel = [GFaGeoFenceManagerSettingModel defaultModel];
    [settingModel setObject:self.apikeyTextField.text forKey:kGeoFenceKeyApiKey];
    [settingModel setObject:self.clientTextField.text forKey:kGeoFenceKeyClientId];
    [settingModel setObject:self.secretTextField.text forKey:kGeoFenceKeySecret];
    [settingModel setObject:[NSNumber numberWithInteger:self.authEnvPullDown.tag] forKey:kGeoFenceKeyAuthEnv];
    
    [settingModel saveToUserDefaults];
}

/* 設定保持クラスの内容を画面入力値に設定 */
- (void)setValueFromGeoFenceManagerSetting_
{
    GFaGeoFenceManagerSettingModel *settingModel = [GFaGeoFenceManagerSettingModel defaultModel];
    self.apikeyTextField.text = [settingModel objectForKey:kGeoFenceKeyApiKey];
    self.clientTextField.text = [settingModel objectForKey:kGeoFenceKeyClientId];
    self.secretTextField.text = [settingModel objectForKey:kGeoFenceKeySecret];
    self.authEnvPullDown.tag = [[settingModel objectForKey:kGeoFenceKeyAuthEnv] integerValue];
}

/* 認証サーバ環境メニューを設定 */
- (void)setupAuthDevMenu_
{
    /* メニュー選択後Block */
    void (^selected)(int) = ^(int authEnv) {
        self.authEnvPullDown.tag = authEnv;
        [self.authEnvPullDown setTitle:[self getAuthEnvMenuTitle_:authEnv] forState:UIControlStateNormal];
    };
    
    /* メニュー選択時コールバックアクション設定 */
    NSMutableArray* actions = [[NSMutableArray alloc] init];
    [actions addObject:[UIAction actionWithTitle:kAuthEnvMenuTitleTest image:nil identifier:nil handler:^(__kindof UIAction* _Nonnull action) {
        selected(AuthEnv_Test);
    }]];
    [actions addObject:[UIAction actionWithTitle:kAuthEnvMenuTitleStg image:nil identifier:nil handler:^(__kindof UIAction* _Nonnull action) {
        selected(AuthEnv_Stg);
    }]];
    [actions addObject:[UIAction actionWithTitle:kAuthEnvMenuTitlePro image:nil identifier:nil handler:^(__kindof UIAction* _Nonnull action) {
        selected(AuthEnv_Pro);
    }]];
    self.authEnvPullDown.menu = [UIMenu menuWithTitle:@"" children:actions];
    
    // 初期表示メニュータイトル設定
    [self.authEnvPullDown setTitle:[self getAuthEnvMenuTitle_:self.authEnvPullDown.tag] forState:UIControlStateNormal];
    self.authEnvPullDown.showsMenuAsPrimaryAction = true;
}

/* 認証サーバ環境メニュータイトルを取得 */
-(NSString *)getAuthEnvMenuTitle_:(NSInteger)authEnv
{
    switch (authEnv) {
        case AuthEnv_Test:
            return kAuthEnvMenuTitleTest;
        case AuthEnv_Stg:
            return kAuthEnvMenuTitleStg;
        case AuthEnv_Pro:
        default:
            return kAuthEnvMenuTitlePro;
    }
}

@end
