//
//  GFaMainTableViewController.m
//
//  Copyright (c) 2013 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "GFaMainTableViewController.h"

#import "GFaManualGeoFenceViewController.h"
#import "GFaLocationManagerViewController.h"
#import <libGeoFenceManager/GeoFenceManager.h>
#import "GFaUtility.h"
#import "GFaServerConnection.h"
#import "GFaAppSettingModel.h"
#import "GFaGeoFenceManagerSettingModel.h"
#import "GFaLocationManagerSettingModel.h"
#import "GFaGeoFenceAreaUpdateTimeModel.h"
#import "GFaCommon.h"

/* トークン項目定義 */
#define kTokenId          @"access_token"
#define kTokenExpireDate  @"expiresDate"

/* セクション定数定義 */
enum {
    kGFaMainTableViewControllerSection_INFO = 0,    /* 情報取得・設定 */
    kGFaMainTableViewControllerSection_CHECK,       /* 確認 */
    kGFaMainTableViewControllerSection_ETC,         /* その他 */
    kGFaMainTableViewControllerSection_MAX,         // MAX
};

@interface GFaMainTableViewController () <UIActionSheetDelegate>
{
    NSMutableDictionary *_tokenInfo;    // トークン情報
    bool searchAreaExecutionFlg;        // エリア検索実行フラグ
}

- (void)showSettingsMenu_;
- (void)doClearAllInformation_;
- (void)showAreaInformationSetting_;
- (void)showNotificationInformationSetting_;
- (void)showManualGeoFenceViewController_;
- (void)showLocationManagerViewController_;
- (void)syncSearchArea_;
- (void)updateSearchArea_;
- (void)requestGetToken_:(void (^)(void))successHandler;
- (void)requestSearchAreaUpdateTime_:(void (^)(NSString *))successHandler;
- (BOOL)requestSearchArea_;

@end

@implementation GFaMainTableViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // 設定情報のロード処理（ソース定義値 or UserDefaults設定値）
        [GFaAppSettingModel defaultModel];
        [GFaLocationManagerSettingModel defaultModel];
        [GFaGeoFenceManagerSettingModel defaultModel];
    }
    
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    _tokenInfo = [NSMutableDictionary dictionary];

    self.title = @"ジオフェンス サンプルアプリ";

    UIBarButtonItem *backButton = [[UIBarButtonItem alloc] init];
    backButton.title = @"戻る";
    self.navigationItem.backBarButtonItem = backButton;

    self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] init]
                                              initWithTitle:@"メニュー"
                                              style:UIBarButtonItemStylePlain
                                              target:self
                                              action:@selector(showSettingsMenu_)];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return kGFaMainTableViewControllerSection_MAX;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSInteger numberOfRowsInSection = 0;
    
    switch (section)
    {
        case kGFaMainTableViewControllerSection_INFO:
            numberOfRowsInSection = 4;
            break;
        case kGFaMainTableViewControllerSection_CHECK:
            numberOfRowsInSection = 2;
            break;
        case kGFaMainTableViewControllerSection_ETC:
            numberOfRowsInSection = 1;
            break;
    }
    
    return numberOfRowsInSection;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    // Configure the cell...
    if ( cell == nil )
    {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
    }
    
    if ( indexPath.section == kGFaMainTableViewControllerSection_INFO )
    {
        if (indexPath.row == 0)
        {
            cell.textLabel.text = @"サーバからエリアを取得";
        }
        else if (indexPath.row == 1)
        {
            cell.textLabel.text = @"エリア情報管理";
        }
        else if (indexPath.row == 2)
        {
            cell.textLabel.text = @"通知条件管理";
        }
        else if (indexPath.row == 3)
        {
            cell.textLabel.text = @"エリア情報/通知条件リセット";
        }
    }
    else if ( indexPath.section == kGFaMainTableViewControllerSection_CHECK )
    {
        if (indexPath.row == 0)
        {
            cell.textLabel.text = @"手入力で確認";
        }
        if (indexPath.row == 1)
        {
            cell.textLabel.text = @"位置測位ライブラリで確認";
        }
    }
    else if ( indexPath.section == kGFaMainTableViewControllerSection_ETC )
    {
        if (indexPath.row == 0)
        {
            cell.textLabel.text = @"バージョン取得";
        }
    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    // エリア検索中の場合は何もしない
    if (searchAreaExecutionFlg) return;
    
    if (indexPath.section == kGFaMainTableViewControllerSection_INFO )
    {
        if (indexPath.row == 0)
        {
            [self syncSearchArea_];
        }
        else if (indexPath.row == 1)
        {
            [self showAreaInformationSetting_];
        }
        else if (indexPath.row == 2)
        {
            [self showNotificationInformationSetting_];
        }
        else if (indexPath.row == 3)
        {
            [self doClearAllInformation_];
        }
    }
    else if (indexPath.section == kGFaMainTableViewControllerSection_CHECK)
    {
        if (indexPath.row == 0)
        {
            
            [self showManualGeoFenceViewController_];
        }
        else if (indexPath.row == 1)
        {
            [self showLocationManagerViewController_];
        }
    }
    else if (indexPath.section == kGFaMainTableViewControllerSection_ETC)
    {
        if (indexPath.row == 0)
        {
            NSString *version = [[GeoFenceManager defaultManager] libraryVersion];
            [GFaUtility showInformationAlert:self title:@"バージョン" message:version];
        }
    }
}

#pragma mark Private

/* メニューアクションシートを表示 */
- (void)showSettingsMenu_
{
    NSString *cancelTitle = @"キャンセル";
    NSString *geofenceSettingsTitle = @"ジオフェンス・ライブラリ設定";
    NSString *appSettingTitle = @"アプリ設定";
    NSString *locationSettingsTitle = @"位置測位・ライブラリ設定";

    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil
                                                                            message:nil
                                                                            preferredStyle:UIAlertControllerStyleActionSheet];
    void (^handler)(UIAlertAction *action) = ^(UIAlertAction *action){
        if ([action.title isEqualToString:cancelTitle]) return;

        UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"SettingStory" bundle:[NSBundle mainBundle]];
        UIViewController *viewController;
        if ([action.title isEqualToString:geofenceSettingsTitle])
        {
            /* ジオフェンス・ライブラリ設定画面 */
            viewController = [storyboard instantiateViewControllerWithIdentifier:@"GFaGeoFenceManagerSettingViewController"];
            GFaAssert(viewController);
        }
        else if ([action.title isEqualToString:appSettingTitle])
        {
            /* アプリ設定画面 */
            viewController = [storyboard instantiateViewControllerWithIdentifier:@"GFaAppSettingViewController"];
            GFaAssert(viewController);
        }
        else if ([action.title isEqualToString:locationSettingsTitle])
        {
            /* 位置測位・ライブラリ設定画面 */
            viewController = [storyboard instantiateViewControllerWithIdentifier:@"GFaLocationManagerSettingViewController"];
            GFaAssert(viewController);
        }
        [self.navigationController pushViewController:viewController animated:YES];
    };

    [alertController addAction:[UIAlertAction actionWithTitle:geofenceSettingsTitle
                                                        style:UIAlertActionStyleDefault
                                                        handler:handler]];
    [alertController addAction:[UIAlertAction actionWithTitle:appSettingTitle
                                                        style:UIAlertActionStyleDefault
                                                        handler:handler]];
    [alertController addAction:[UIAlertAction actionWithTitle:locationSettingsTitle
                                                        style:UIAlertActionStyleDefault
                                                        handler:handler]];
    [alertController addAction:[UIAlertAction actionWithTitle:cancelTitle
                                                        style:UIAlertActionStyleCancel
                                                        handler:handler]];

    [self.navigationController presentViewController:alertController animated:YES completion:nil];
}

/* 「エリア情報/通知条件のリセット」ダイアログを表示 */
-(void)doClearAllInformation_
{
    NSString* title = @"エリア情報/通知条件のリセット";
    NSString* message = @"すべてのエリア情報・通知条件を削除しますか？";
    [GFaUtility showConfirmAlert:self title:title message:message leftButtonTitle:@"キャンセル" rightButtonTitle:@"OK" leftHandler:^{
        // none
    } rightHandler:^{
        [[GeoFenceManager defaultManager] clearAllInformation];
        [GFaUtility showInformationAlert:self title:title message:@"すべてのエリア情報・通知条件を削除しました"];
    }];
}

/* エリア情報管理画面へ遷移 */
- (void)showAreaInformationSetting_
{
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"AreaInformation" bundle:[NSBundle mainBundle]];
    GFaAssert(storyboard);
    UIViewController *viewController = [storyboard instantiateInitialViewController];
    GFaAssert(viewController);
    viewController.title = @"エリア情報管理";
    [self.navigationController pushViewController:viewController animated:YES];
}

/* 通知条件管理画面へ遷移 */
- (void)showNotificationInformationSetting_
{
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"NotificationCondition" bundle:[NSBundle mainBundle]];
    GFaAssert(storyboard);
    UIViewController *viewController = [storyboard instantiateInitialViewController];
    GFaAssert(viewController);
    viewController.title = @"通知条件管理";
    [self.navigationController pushViewController:viewController animated:YES];
}

/* 手入力で確認画面へ遷移 */
- (void)showManualGeoFenceViewController_
{
    GFaManualGeoFenceViewController *viewController = [[GFaManualGeoFenceViewController alloc]
                                                       initWithNibName:@"GFaManualGeoFenceViewController"
                                                       bundle:nil];
    GFaAssert(viewController);
    [self.navigationController pushViewController:viewController animated:YES];
}

/* 位置測位ライブラリで確認画面へ遷移 */
- (void)showLocationManagerViewController_
{
    GFaLocationManagerViewController *viewController = [[GFaLocationManagerViewController alloc]
                                                        initWithNibName:@"GFaLocationManagerViewController"
                                                        bundle:nil];
    GFaAssert(viewController);
    [self.navigationController pushViewController:viewController animated:YES];
}

/* サーバとエリア情報を同期 */
- (void)syncSearchArea_
{
    searchAreaExecutionFlg = YES;

    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
        // 認証トークンなし、または、トークン有効期間を超えた場合、認証トークン取得
        NSDate *now = [NSDate date];
        if (self->_tokenInfo[kTokenId] != nil &&
            [now compare:self->_tokenInfo[kTokenExpireDate]] <= NSOrderedSame)
        {
            // エリア情報取得
            [self updateSearchArea_];
        }
        else
        {
            // 認証トークン取得
            [self requestGetToken_:^{
                // エリア情報取得
                [self updateSearchArea_];
            }];
        }
        self->searchAreaExecutionFlg = NO;
    });
}

/* サーバからエリア更新日時を取得し、更新確認ダイアログを表示 */
- (void)updateSearchArea_
{
    [self requestSearchAreaUpdateTime_:^(NSString * updateTime) {
        // 前回サーバから取得したエリア更新日時をロード
        GFaGeoFenceAreaUpdateTimeModel *model = [GFaGeoFenceAreaUpdateTimeModel defaultModel];
        NSString *localUpdateTime = [model get];
        if (localUpdateTime == nil) {
            localUpdateTime = @"なし";
        }
        NSString *message = [NSString stringWithFormat:@"サーバのエリア情報をジオフェンスライブラリに反映しますか？\n\nサーバ側のエリア更新日時\n%@\nローカル側のエリア更新日時\n%@", updateTime, localUpdateTime];

       // エリア情報の反映確認ダイアログを表示
        self->searchAreaExecutionFlg = YES;
        [GFaUtility showConfirmAlert:self title:@"エリア情報の反映確認" message:message leftButtonTitle:@"いいえ" rightButtonTitle:@"はい" leftHandler:^{
                self->searchAreaExecutionFlg = NO;
            } rightHandler:^{
                dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
                    // エリア検索API実行
                    Boolean isSuccess = [self requestSearchArea_];
                    // エリア検索に成功した場合は、サーバから取得したエリア更新日時を永続化
                    if (isSuccess)
                    {
                        GFaGeoFenceAreaUpdateTimeModel *model = [GFaGeoFenceAreaUpdateTimeModel defaultModel];
                        [model set:updateTime];
                    }
                    self->searchAreaExecutionFlg = NO;
                });
            }];
    }];
}

/* トークン発行サーバから認証トークンを取得 */
- (void)requestGetToken_:(void (^)(void))successHandler
{
    NSError *error = nil;
    NSHTTPURLResponse *response = nil;
    NSDictionary *data = nil;

    GFaAppSettingModel *appModel = [GFaAppSettingModel defaultModel];
    NSString *clientId = [appModel objectForKey:kAppKeyClientId];
    NSString *secret = [appModel objectForKey:kAppKeySecret];
    NSString *tokenUrl = [appModel objectForKey:kAppKeyTokenURL];

    // トークン取得通信
    GFaServerConnection *server = [[GFaServerConnection alloc]init];
    [server requestGetToken:tokenUrl clientId:clientId secret:secret response:&response data:&data error:&error];
    if ([server isResponseOK:response] == YES)
    {
        NSString *tokenId = [data objectForKey:@"access_token"];
        NSNumber *expiresIn = [data objectForKey:@"expires_in"];
        if (tokenId != nil && [expiresIn intValue] > 0)
        {
            [self->_tokenInfo setObject:tokenId forKey:kTokenId];
            // トークン有効期限（日時）を設定
            NSDate *expiresDate = [NSDate dateWithTimeIntervalSinceNow:expiresIn.intValue];
            [self->_tokenInfo setObject:expiresDate forKey:kTokenExpireDate];

            successHandler();
        }
    }
    else
    {
        NSLog(@"トークン取得失敗 [レスポンスコード：%ld エラーコード:%ld 詳細:%@]", response.statusCode, error.code, error.localizedDescription);
        [GFaUtility showInformationAlert:self title:@"トークン取得失敗" message:@""];
    }
}

/* エリア情報サーバからエリア更新日時を取得 */
- (void)requestSearchAreaUpdateTime_:(void (^)(NSString *))successHandler
{
    NSError *error = nil;
    NSHTTPURLResponse *response = nil;
    NSDictionary *data = nil;

    GFaAppSettingModel *appModel = [GFaAppSettingModel defaultModel];
    NSString *apikey = [appModel objectForKey:kAppKeyApiKey];
    NSString *searchAreaUpdateTimeURL = [appModel objectForKey:kAppKeySearchAreaUpdateTimeURL];

    // エリア更新日時検索通信
    GFaServerConnection *server = [[GFaServerConnection alloc]init];
    [server requestAreaInfoServer:searchAreaUpdateTimeURL tokenId:_tokenInfo[kTokenId] apikey:apikey json:@"{}" response:&response data:&data error:&error];
    if ([server isResponseOK:response] == YES)
    {
        NSDictionary *res = [data objectForKey:@"response"];
        NSDictionary *status = [res objectForKey:@"status"];
        NSInteger returnCode = [[status objectForKey:@"returnCode"]intValue];
        if (returnCode == AreaServerResponseReturnCode_Success)
        {
            NSDictionary *result = [res objectForKey:@"result"];
            successHandler([result objectForKey:@"updateTime"]);
            return;
        }
        NSLog(@"エリア更新日時取得失敗 [レスポンスJSON:%@]", data);
    }
    else
    {
        NSLog(@"エリア更新日時取得失敗 [レスポンスコード：%ld エラーコード:%ld 詳細:%@]", response.statusCode, error.code, error.localizedDescription);
    }
    [GFaUtility showInformationAlert:self title:@"エリア更新日時取得失敗" message:@""];
}

/* エリア情報サーバからエリアを取得 */
- (BOOL)requestSearchArea_
{
    GFaAppSettingModel *appModel = [GFaAppSettingModel defaultModel];
    NSString *apikey = [appModel objectForKey:kAppKeyApiKey];
    NSString *searchAreaURL = [appModel objectForKey:kAppKeySearchAreaURL];

    GFaServerConnection *server = [[GFaServerConnection alloc]init];
    NSMutableArray *areaList = [NSMutableArray array];
    NSMutableArray *conditionList = [NSMutableArray array];
    int counter = 0;
    int offset = 1;

    while (1)
    {
        @autoreleasepool {
            NSError *error = nil;
            NSHTTPURLResponse *response = nil;
            NSDictionary *data = nil;

            NSString *json = [NSString stringWithFormat:@"{\"limit\":%d, \"offset\":%d}", kSearchAreaLimit, offset];

            // エリア検索通信
            [server requestAreaInfoServer:searchAreaURL tokenId:_tokenInfo[kTokenId] apikey:apikey json:json response:&response data:&data error:&error];
            if ([server isResponseOK:response] == YES)
            {
                NSDictionary *res = [data objectForKey:@"response"];
                NSDictionary *status = [res objectForKey:@"status"];
                NSInteger returnCode = [[status objectForKey:@"returnCode"]intValue];
                if (returnCode == AreaServerResponseReturnCode_Success)
                {
                    NSDictionary *result = [res objectForKey:@"result"];
                    NSArray *areas = [result objectForKey:@"areas"];
                    int limitBeforeCount = [[result valueForKey:@"limitBeforeCount"]intValue];
                    for (NSDictionary *area in areas)
                    {
                        NSString *areaGroupId =  [area objectForKey:@"areaGroupId"];
                        NSString *areaId =  [area objectForKey:@"areaId"];
                        NSString *areaShape = [GFaUtility dictionaryToJson:[area objectForKey:@"areaShape"]];
                        NSString *uniqueAreaId = [NSString stringWithFormat:@"%@#%@",areaGroupId, areaId];

                        // エリア情報
                        NSMutableDictionary *areaInfo = [NSMutableDictionary dictionary];
                        [areaInfo setObject:uniqueAreaId forKey:GFlResponseKeyAid];
                        [areaInfo setObject:areaShape forKey:GFlResponseKeyGinf];
                        [areaInfo setObject:[NSNumber numberWithInt:1] forKey:GFlResponseKeyInfnum];
                        [areaInfo setObject:[NSNumber numberWithInt:1]  forKey:GFlResponseKeyOutfnum];
                        [areaList addObject:areaInfo];

                        // 通知条件（IN）
                        NSMutableDictionary *condInInfo = [NSMutableDictionary dictionary];
                        [condInInfo setObject:[NSNumber numberWithInt:++counter] forKey:GFlResponseKeyNid];
                        [condInInfo setObject:uniqueAreaId forKey:GFlResponseKeyAid];
                        [condInInfo setObject:[NSNumber numberWithInt:GFlResponseKeyNtmg_in] forKey:GFlResponseKeyNtmg];
                        [conditionList addObject:condInInfo];

                        // 通知条件（OUT）
                        NSMutableDictionary *condOutInfo = [NSMutableDictionary dictionary];
                        [condOutInfo setObject:[NSNumber numberWithInt:++counter] forKey:GFlResponseKeyNid];
                        [condOutInfo setObject:uniqueAreaId forKey:GFlResponseKeyAid];
                        [condOutInfo setObject:[NSNumber numberWithInt:GFlResponseKeyNtmg_out] forKey:GFlResponseKeyNtmg];
                        [conditionList addObject:condOutInfo];
                    }
                    // 全てのエリア情報が取得できた時点でループを抜ける
                    offset += kSearchAreaLimit;
                    if (limitBeforeCount < offset) break;
                    
                    // 次のエリア情報を取得
                    continue;
                }
                else
                {
                    NSLog(@"エリア検索失敗 [レスポンスJSON:%@]", data);
                }
            }
            else
            {
                NSLog(@"エリア検索失敗 [レスポンスコード：%ld エラーコード:%ld 詳細:%@]", response.statusCode, error.code, error.localizedDescription);
            }
            [GFaUtility showInformationAlert:self title:@"エリア検索失敗" message:@""];
            return NO;
        }
    }

    GeoFenceManager *manager = [GeoFenceManager defaultManager];
    // ジオフェンスライブラリ登録済みのエリア情報を全クリア
    [manager clearAllInformation];
    // ジオフェンスライブラリにエリア情報サーバから取得したエリアを登録
    NSArray *errorAreaList = [manager addAreaInformationForGeoJson:areaList];
    if ([errorAreaList count] > 0)
    {
        NSLog(@"エリア検索失敗 [エリア情報エラーリスト：%@]", errorAreaList);
        [GFaUtility showInformationAlert:self title:@"エリア検索失敗" message:@""];
        return NO;
    }
    // ジオフェンスライブラリに通知条件（IN/OUT）を登録
    NSArray *errorConditionList = [manager addNotificationConditionInformation:conditionList];
    if ([errorConditionList count] > 0)
    {
        NSLog(@"エリア検索失敗 [通知条件エラーリスト：%@]", errorConditionList);
        [GFaUtility showInformationAlert:self title:@"エリア検索失敗" message:@""];
        return NO;
    }

    [GFaUtility showInformationAlert:self title:@"エリア取得成功" message:@""];
    return YES;
}

@end
