//
//  GFaManualGeoFenceViewController.m
//
//  Copyright (c) 2013 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "GFaManualGeoFenceViewController.h"

#import <libGeoFenceManager/GeoFenceManager.h>
#import "GFaGeoFenceManagerSettingModel.h"
#import "GFaCommon.h"

@interface GFaManualGeoFenceViewController ()

@property (nonatomic, strong) UIDatePicker *datePicker;
@property (nonatomic, strong) NSDate *checkDate;

- (void)checkLocation_:(NSDate *)timeStamp latitude:(NSString *)latitude longitude:(NSString *)longitude horizontalAccuracy:(NSString *)horizontalAccuracy;
- (NSString *)stringFromDate_:(NSDate *)date;
- (void)appendString_:(NSString *)string;
- (void)hydeKeyBoard_;
- (void)pushClearButton_:(id)sender;
- (void)updateDateButtonTitle_:(NSDate *)date;
- (void)showDatePicker_;
- (void)appendParams_:(NSDate *)timeStamp longitude:(NSString *)lon latitude:(NSString *)lat horizontalAccuracy:(NSInteger)hAccuracy;
- (void)appendCheckLocationResult_:(NSNumber *)nid areaId:(NSString *)aid notifyTiming:(NSNumber *)ntmg;

@end

@implementation GFaManualGeoFenceViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.title = @"手入力で確認";

    // Do any additional setup after loading the view from its nib.
    UITapGestureRecognizer *tapRevog = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hydeKeyBoard_)];
    [self.view addGestureRecognizer:tapRevog];
    
    [self pushClearButton_:nil];
    
    UIBarButtonItem *buttonItem = [[UIBarButtonItem alloc] initWithTitle:@"クリア"
                                                                   style:UIBarButtonItemStylePlain
                                                                  target:self
                                                                  action:@selector(pushClearButton_:)];
    self.navigationItem.rightBarButtonItem = buttonItem;
    
    self.datePicker = [[UIDatePicker alloc] init];
    self.checkDate = self.datePicker.date;
    [self updateDateButtonTitle_:self.checkDate];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)pushCheckButton:(id)sender
{
    [self hydeKeyBoard_];
    
    NSString *lat = [self.latitudeTextField text];
    NSString *lon = [self.longitudeTextField text];
    NSString *accuracy = [self.accuracyTextField text];
    NSDate *date = self.checkDate;
    
    if ( lat.length > 0 && lon.length > 0 && accuracy.length > 0 && date != nil )
    {
        // 非同期スレッドでジオフェンス内外判定を実施
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
            @synchronized (self) {
                [self checkLocation_:date latitude:lat longitude:lon horizontalAccuracy:accuracy];
            }
        });
    }
}
- (IBAction)pushResetButton:(id)sender
{
    [[GeoFenceManager defaultManager] resetGeoFenceStatus];
    [self appendString_:@"エリア滞在状態と通知状態をリセットしました\n\n"];
}

- (IBAction)pushDateButton:(id)sender
{
    [self hydeKeyBoard_];
    [self showDatePicker_];
}

// iOS8以降のPickerViewのDone押下時
- (IBAction)pushPickerDoneButton:(id)sender
{
    self.checkDate = m_Picker.date;
    [self updateDateButtonTitle_:self.checkDate];
    m_PickerView.hidden = YES;
    [self.navigationController.navigationBar setHidden:NO];
}

// iOS8以降のPickView画面の背景をタップした際にPickerを閉じる
- (IBAction)pushClosePickerButton:(id)sender
{
    self.checkDate = m_Picker.date;
    [self updateDateButtonTitle_:self.checkDate];
    m_PickerView.hidden = YES;
    [self.navigationController.navigationBar setHidden:NO];
}

#pragma mark Private

/* 位置測位結果からジオフェンス内外判定 */
- (void)checkLocation_:(NSDate *)timeStamp latitude:(NSString *)latitude longitude:(NSString *)longitude horizontalAccuracy:(NSString *)horizontalAccuracy
{
    NSError *error = nil;
    GFaGeoFenceManagerSettingModel *gmSettingModel = [GFaGeoFenceManagerSettingModel defaultModel];
    
    NSInteger hAccuracy = [horizontalAccuracy integerValue];

    GeoFenceManager *manager = [GeoFenceManager defaultManager];
    NSArray *checkResult = [manager checkLocation:[gmSettingModel objectForKey:kGeoFenceKeyApiKey]
                                         clientId:[gmSettingModel objectForKey:kGeoFenceKeyClientId]
                                        secret:[gmSettingModel objectForKey:kGeoFenceKeySecret]
                                        authEnv:[[gmSettingModel objectForKey:kGeoFenceKeyAuthEnv] integerValue]
                                        timeStamp:timeStamp
                                        latitude:[latitude doubleValue]
                                        longitude:[longitude doubleValue]
                                        horizontalAccuracy:hAccuracy
                                        error:&error];
    
    if (error == nil)
    {
        if (checkResult.count > 0)
        {
            for ( NSDictionary *info in checkResult )
            {
                NSDictionary *notificationConditionInfo = [info objectForKey:GFlResponseTagKeyCondition];
                NSNumber *notificationID = [notificationConditionInfo objectForKey:GFlResponseKeyNid];
                NSNumber *notifyTiming = [notificationConditionInfo objectForKey:GFlResponseKeyNtmg];

                NSDictionary *areaInfo = [info objectForKey:GFlResponseTagKeyArea];
                NSString *areaID = [areaInfo objectForKey:GFlResponseKeyAid];

                [self appendParams_:timeStamp longitude:longitude latitude:latitude horizontalAccuracy:hAccuracy];
                [self appendCheckLocationResult_:notificationID areaId:areaID notifyTiming:notifyTiming];
            }
        }
        else
        {
            [self appendParams_:timeStamp longitude:longitude latitude:latitude horizontalAccuracy:hAccuracy];
        }
    }
    else
    {
        [self appendString_:@"エリア内外判定に失敗しました\n\n"];
        NSLog(@"エリア内外判定失敗 [エラーコード:%ld 詳細:%@]", error.code, error.localizedDescription);
    }
}

- (NSString *)stringFromDate_:(NSDate *)date
{
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"US"];
    
    [formatter setDateFormat:@"yyyy/MM/dd HH:mm:ss"];
    [formatter setLocale:locale];
    [formatter setTimeZone:[NSTimeZone systemTimeZone]];
    
    return [formatter stringFromDate:date];
}

- (void)appendString_:(NSString *)string
{
    dispatch_async(dispatch_get_main_queue(), ^{
        NSString *newString = [string stringByAppendingString:[self.textView text]];
        [self.textView setText:newString];
    });
}

- (void)hydeKeyBoard_
{
    NSArray *list = @[
        self.latitudeTextField,
        self.longitudeTextField,
        self.accuracyTextField,
    ];
    
    for ( UITextField *textField in list )
    {
        [textField resignFirstResponder];
    }
}

- (void)pushClearButton_:(id)sender
{
    [self.textView setText:@""];
}

- (void)updateDateButtonTitle_:(NSDate *)date
{
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"US"];
    
    [formatter setDateFormat:@"yyyy/MM/dd HH:mm"];
    [formatter setLocale:locale];
    [formatter setTimeZone:[NSTimeZone systemTimeZone]];
    
    NSString *title = [formatter stringFromDate:date];
    
    [self.dateButton setTitle:title forState:UIControlStateNormal];
}

// DatePickerの表示
- (void)showDatePicker_
{
    m_PickerView.hidden = NO;
    [self.navigationController.navigationBar setHidden:YES];
}

/* パラメータを出力 */
- (void)appendParams_:(NSDate *)timeStamp longitude:(NSString *)lon latitude:(NSString *)lat horizontalAccuracy:(NSInteger)hAccuracy
{
    NSString *msg = @"日時:[%@]\n緯度:[%@]\n経度:[%@]\n誤差:[%ld]\n\n";
    NSString *date = [self stringFromDate_:timeStamp];
    NSString *info = [NSString stringWithFormat:msg, date, lat, lon, hAccuracy];
    [self appendString_:info];
}

/* 内外判定結果を出力 */
- (void)appendCheckLocationResult_:(NSNumber *)nid areaId:(NSString *)aid notifyTiming:(NSNumber *)ntmg
{
    NSString *msg = @"*** 通知条件ID:[%@] エリアID:[%@] 内外判定:[%@] ***\n";
    NSString *tmg = [ntmg intValue] == GFlResponseKeyNtmg_in ? @"内" : @"外";
    NSString *info = [NSString stringWithFormat:msg, [nid stringValue], aid, tmg];
    [self appendString_:info];
}

@end

