//
//  GFaSettingModel.m
//
//  Copyright (c) 2022 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "GFaSettingModel.h"

#import <libGeoFenceManager/GeoFenceManager.h>
#import "GFaCommon.h"

@interface GFaSettingModel ()

/* UserDefaultキー名 */
@property (nonatomic, strong) NSString *userDefaultsKey;

@end

@implementation GFaSettingModel

/* 初期処理 */
- (id) init {
    return [self initModel:nil];
}

/* 初期処理（UserDefaultキー指定） */
- (id)initModel:(NSString *)userDefaultsKey
{
    self.userDefaultsKey = userDefaultsKey;

    self = [super init];
    if ( self )
    {
        self.settingInfo = [NSMutableDictionary dictionary];
        if ( [self restoreFromUserDefaults] == NO )
        {
            [self.settingInfo addEntriesFromDictionary:[self defaultInfo]];
            // ソースに定義されている初期値を永続化
            [self saveToUserDefaults];
        }
    }
    return self;
}

/* 設定値変更 */
- (void)setObject:(id)object forKey:(NSString *)key
{
    do {
        GFaBreakIf(key.length == 0);
        BOOL remove = NO;
        if ( object == nil ||
            ([object isKindOfClass:[NSString class]] && [object length] == 0) )
        {
            remove = YES;
        }
        
        if ( remove == YES )
        {
            [self.settingInfo removeObjectForKey:key];
        }
        else
        {
            [self.settingInfo setObject:object forKey:key];
        }
    } while(0);
}

/* 設定値取得 */
- (id)objectForKey:(NSString *)key
{
    return [self.settingInfo objectForKey:key];
}

/* リセット */
- (void)resetSetting
{
    [self.settingInfo removeAllObjects];
    [self.settingInfo addEntriesFromDictionary:[self defaultInfo]];
}

/* UserDefaultsに登録されている設定値から復元 */
- (BOOL)restoreFromUserDefaults
{
    BOOL didRestored = NO;
    
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSDictionary *info = [userDefaults dictionaryForKey:self.userDefaultsKey];
    if ( info != nil )
    {
        [self.settingInfo addEntriesFromDictionary:info];
        didRestored = YES;
    }
    
    return didRestored;
}

/* UserDefaultsに設定値を登録 */
- (BOOL)saveToUserDefaults
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.settingInfo forKey:self.userDefaultsKey];
    
    return [userDefaults synchronize];
}

/* ソース内で定義されている設定値（初期値）を取得 ※取得値は継承先クラスで設定 */
- (NSDictionary *)defaultInfo
{
    return nil;
}

@end
