//
//  GFaUtility.m
//
//  Copyright (c) 2014 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "GFaUtility.h"

@implementation GFaUtility

/* 日付フォーマット変更 */
+ (NSDate *)dateFromFormatFromServer:(NSString *)dateFormatStr
{
    NSDate *date = nil;
    
    do {
        if ( dateFormatStr.length == 0 ) break;
        
        NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"US"];
        NSTimeZone *timezone = [NSTimeZone systemTimeZone];
        
        NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
        [formatter setDateFormat:@"yyyyMMddHHmm"];
        [formatter setTimeZone:timezone];
        [formatter setLocale:locale];
        
        date = [formatter dateFromString:dateFormatStr];
    } while(0);
    
    return date;
}

/* 情報ダイアログを表示 */
+ (void)showInformationAlert:(UIViewController *)viewController title:(NSString *)title message:(NSString *)message
{
    dispatch_async(dispatch_get_main_queue(), ^{
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title
                                                                            message:message
                                                                            preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil]];
        [viewController presentViewController:alertController animated:YES completion:nil];
    });
}

/* 確認ダイアログを表示 */
+ (void)showConfirmAlert:(UIViewController *)viewController title:(NSString *)title message:(NSString *)message leftButtonTitle:(NSString *)leftTitle  rightButtonTitle:(NSString *)rightTitle leftHandler:(void(^)(void))leftBlock rightHandler:(void(^)(void))rightBlock
{
    dispatch_async(dispatch_get_main_queue(), ^{
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title
                                                                            message:message
                                                                            preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:leftTitle style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action){
            leftBlock();
        }]];
        [alertController addAction:[UIAlertAction actionWithTitle:rightTitle style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action){
            rightBlock();
        }]];
        [viewController presentViewController:alertController animated:YES completion:nil];
    });
}

/* 入力ダイアログを表示 */
+ (void)showInputAlert:(UIViewController *)viewController title:(NSString *)title message:(NSString *)message text:(NSString *)inputText cancelButtonTitle:(NSString *)cancelTitle otherButtonTitle:(NSString *)otherTitle keyboardType:(UIKeyboardType)keyboardType otherHandler:(void(^)(NSString *))otherBlock;
{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title
                                                                             message:message
                                                                             preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *textField) {
        textField.text = inputText;
        textField.keyboardType = keyboardType;
        [textField becomeFirstResponder];
    }];
    [alertController addAction:[UIAlertAction actionWithTitle:cancelTitle style:UIAlertActionStyleCancel handler:nil]];
    [alertController addAction:[UIAlertAction actionWithTitle:otherTitle style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        otherBlock(alertController.textFields[0].text);
    }]];
    [viewController presentViewController:alertController animated:YES completion:nil];
}

/* dictionaryをJSON文字列に変換 */
+ (NSString *) dictionaryToJson:(NSDictionary *)dictionary
{
    NSError *error = nil;
    NSData *json = [NSJSONSerialization dataWithJSONObject:dictionary options:2 error:&error];
    return [[NSString alloc]initWithData:json encoding:NSUTF8StringEncoding];
}

@end
